@@ -55,7 +55,7 @@ except ImportError:
             message_type=Gtk.MessageType.ERROR,
             buttons=Gtk.ButtonsType.OK
         )
-        dlg.set_markup(_('You need to install the Python simplejson module.'))
+        dlg.set_markup(_(u'You need to install the Python simplejson module.'))
         dlg.run()
         dlg.destroy()
         raise
@@ -124,7 +124,7 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
 
         self.reset()
         container.show_all()
-        self.totem.add_sidebar_page("jamendo", _("Jamendo"), container)
+        self.totem.add_sidebar_page("jamendo", _(u"Jamendo"), container)
 
     def do_deactivate(self):
         """
@@ -138,16 +138,17 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
         This code must be independent from the rest of the plugin. FIXME: bgo#624073
         """
         builder = Totem.plugin_load_interface ('jamendo', 'jamendo.ui', True, None, self)
-        print builder
         config_widget = builder.get_object ('config_widget')
         config_widget.connect ('destroy', self.on_config_widget_destroy)
-        print config_widget
         format = self.settings.get_enum ('format')
-        num_per_page = self.settings.get_int ('num-per-page')
+        num_per_page = self.settings.get_value ('num-per-page').get_uint32 ()
 
+        # Set up the "format" combo box. We can't use g_settings_bind() here, as it won't automatically convert between enums and ints. To do so,
+        # we'd need to use g_settings_bind_with_mapping(), but that isn't introspectable. We have to handle the binding manually.
         combo = builder.get_object('preferred_format_combo')
         combo.set_active(format)
-        self.settings.bind ('format', combo, 'active', Gio.SettingsBindFlags.DEFAULT)
+        combo.connect ('changed', self.on_format_combo_changed)
+        self.settings.connect ('changed::format', self.on_format_setting_changed, combo)
 
         spinbutton = builder.get_object('album_num_spinbutton')
         spinbutton.set_value(num_per_page)
@@ -155,6 +156,18 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
 
         return config_widget
 
+    def on_format_combo_changed (self, combo):
+        """
+        Called when the "format" preference combo box value is changed.
+        """
+        self.settings.set_enum ('format', combo.get_active ())
+
+    def on_format_setting_changed (self, settings, key, combo):
+        """
+        Called for the "format" preference combo box when the corresponding GSettings value is changed.
+        """
+        combo.set_active (self.settings.get_enum ('format'))
+
     def on_format_changed (self, settings, key):
         JamendoService.AUDIO_FORMAT = self.settings.get_enum ('format')
 
@@ -197,7 +210,9 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
         """
         self.current_treeview = self.treeviews[0]
         for w in self.treeviews:
-            w.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
+            selection = w.get_selection ()
+            selection.set_mode(Gtk.SelectionMode.MULTIPLE)
+            selection.connect ('changed', self.on_treeview_selection_changed)
 
             # build pixbuf column
             cell = Gtk.CellRendererPixbuf()
@@ -207,8 +222,7 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
 
             # build description column
             cell = Gtk.CellRendererText()
-            cell.set_property('wrap-mode', Pango.WrapMode.WORD)
-            cell.set_property('wrap-width', 30)
+            cell.set_property('ellipsize', Pango.EllipsizeMode.END)
             col = Gtk.TreeViewColumn(cell_renderer=cell, markup=2)
             col.set_expand(True)
             w.append_column(col)
@@ -243,23 +257,23 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
                     8, 1, 1)
         # format title
         title  = '<b>%s</b>\n' % self._format_str(album['name'])
-        title += _('Artist: %s') % self._format_str(album['artist_name'])
+        title += _(u'Artist: %s') % self._format_str(album['artist_name'])
         # format duration
         dur = self._format_duration(album['duration'])
         # format tooltip
         try:
             # Translators: this is the release date of an album in Python strptime format
-            release = time.strptime(album['dates']['release'][0:10], _('%Y-%m-%d'))
+            release = time.strptime(album['dates']['release'][0:10], _(u'%Y-%m-%d'))
             # Translators: this is the release time of an album in Python strftime format
-            release = time.strftime(_('%x'), release)
+            release = time.strftime(_(u'%x'), release)
         except:
             release = ''
         tip = '\n'.join([
             '<b>%s</b>' % self._format_str(album['name']),
-            _('Artist: %s') % self._format_str(album['artist_name']),
-            _('Genre: %s') % self._format_str(album['genre']),
-            _('Released on: %s') % release,
-            _('License: %s') % self._format_str(album['license'][0]),
+            _(u'Artist: %s') % self._format_str(album['artist_name']),
+            _(u'Genre: %s') % self._format_str(album['genre']),
+            _(u'Released on: %s') % release,
+            _(u'License: %s') % self._format_str(album['license'][0]),
         ])
         # append album row
         parent = treeview.get_model().append(None,
@@ -272,16 +286,16 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
             # track title
             # Translators: this is the title of a track in Python format
             # (first argument is the track number, second is the track title)
-            tt = ('<small>%s</small>' % _('%02d. %s')) % \
+            tt = ('<small>%s</small>' % _(u'%02d. %s')) % \
                 (i+1, self._format_str(track['name']))
             # track duration
             td = self._format_duration(track['duration'])
             # track tooltip
             tip = '\n'.join([
                 '<b>%s</b>' %  self._format_str(track['name']),
-                _('Album: %s') % self._format_str(album['name']),
-                _('Artist: %s') % self._format_str(album['artist_name']),
-                _('Duration: %s') % td,
+                _(u'Album: %s') % self._format_str(album['name']),
+                _(u'Artist: %s') % self._format_str(album['artist_name']),
+                _(u'Duration: %s') % td,
             ])
             # append track
             treeview.get_model().append(parent, [track, icon, tt, td, tip])
@@ -311,9 +325,9 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
         enqueue_and_play.
         """
         if mode == 'replace':
-            self.totem.action_remote(Totem.RemoteCommand.REPLACE, t['stream'])
+            self.totem.action_remote(Totem.RemoteCommand.REPLACE, t['stream'].encode ('UTF-8'))
         elif mode == 'enqueue':
-            self.totem.action_remote(Totem.RemoteCommand.ENQUEUE, t['stream'])
+            self.totem.action_remote(Totem.RemoteCommand.ENQUEUE, t['stream'].encode ('UTF-8'))
 
     def fetch_albums(self, pn=1):
         """
@@ -338,7 +352,7 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
         self.progressbars[tab_index].show()
         self.progressbars[tab_index].set_fraction(0.0)
         self.progressbars[tab_index].set_text(
-            _('Fetching albums, please wait...')
+            _(u'Fetching albums, please wait…')
         )
         lcb = (self.on_fetch_albums_loop, self.current_treeview)
         dcb = (self.on_fetch_albums_done, self.current_treeview)
@@ -393,13 +407,13 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
                 except:
                     reason = str(exc)
             reason = reason.capitalize()
-            msg = _('Failed to connect to Jamendo server.\n%s.') % reason
+            msg = _(u'Failed to connect to Jamendo server.\n%s.') % reason
         elif hasattr(exc, 'code'):
-            msg = _('The Jamendo server returned code %s.') % exc.code
+            msg = _(u'The Jamendo server returned code %s.') % exc.code
         else:
             msg = str(exc)
 
-        self.totem.action_error(_('An error occurred while fetching albums.'), msg)
+        self.totem.action_error(_(u'An error occurred while fetching albums.'), msg)
 
     def on_search_entry_activate(self, *args):
         """
@@ -457,27 +471,31 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
         Called when the user clicked on a treeview element.
         """
         try:
-            if evt.button == 3:
-                path = tv.get_path_at_pos(int(evt.x), int(evt.y))
+            if evt.button.button == 3:
+                (path, _, _, _) = tv.get_path_at_pos(int(evt.x), int(evt.y))
                 sel  = tv.get_selection()
                 (rows, _) = sel.get_selected_rows()
-                if path[1] not in rows[1]:
+                if path not in rows:
                     sel.unselect_all()
-                    sel.select_path(path[1])
+                    sel.select_path(path)
                 tv.grab_focus()
-                self.popup.popup(None, None, None, evt.button, evt.time)
+                self.popup.popup_for_device(None, None, None, None, None, evt.button.button, evt.time)
                 return True
-            coords = evt.get_coords()
-            _, path, c, x, y = tv.get_path_at_pos(int(coords[0]), int(coords[1]))
+
+            (_, event_x, event_y) = evt.get_coords()
+            path, c, x, y = tv.get_path_at_pos(int(event_x), int(event_y))
             if (path.get_depth() == 1):
                 if tv.row_expanded(path):
                     tv.collapse_row(path)
                 else:
                     tv.expand_row(path, False)
-            self.album_button.set_sensitive(True)
         except:
             pass
 
+    def on_treeview_selection_changed (self, selection):
+        (rows, _) = selection.get_selected_rows ()
+        self.album_button.set_sensitive (len (rows) > 0)
+
     def on_previous_button_clicked(self, *args):
         """
         Called when the user clicked the previous button.
@@ -548,11 +566,15 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
         sel = self.current_treeview.get_selection()
         (rows, model) = sel.get_selected_rows()
         for row in rows:
+            it = model.get_iter(row)
+
+            # Return the parent node if root == true
             if root:
-                it = model.get_iter((row[0],))
-            else:
-                it = model.get_iter(row)
-            elt = model.get(it, 0)[0]
+                parent_iter = model.iter_parent (it)
+                if parent_iter != None:
+                    it = parent_iter
+
+            elt = model.get_value(it, 0)
             if elt not in ret:
                 ret.append(elt)
         return ret
@@ -593,10 +615,10 @@ class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasGtk.Configurable):
             if secs >= 3600:
                 # Translators: time formatting (in Python strftime format) for the Jamendo plugin
                 # for times longer than an hour
-                return time.strftime(_('%H:%M:%S'), time.gmtime(secs))
+                return time.strftime(_(u'%H:%M:%S'), time.gmtime(secs))
             # Translators: time formatting (in Python strftime format) for the Jamendo plugin
             # for times shorter than an hour
-            return time.strftime(_('%M:%S'), time.gmtime(secs))
+            return time.strftime(_(u'%M:%S'), time.gmtime(secs))
         except:
             return ''
 
